program diff
  !  
  !       Este codigo resuelve la ecuacion de difusion 
  !       dF/dt   =  k * d^2 F/dx^2 
  !       con k constante. 
  !       Utiliza un esquema temporal explicito
  !       y centrado en el espacio a segundo o cuarto orden.
  !
  !--------------------------------------------------------------------
  !      Dejo los ingredientes necesarios para generalizar el esquema 
  !      temporal.
  !
  implicit none
  !
  real, allocatable, dimension(:) :: F, DF, DDF, r
  real      ::    dt, k, t, rmin, rmax
  real      ::    alfa, beta, sum
  integer   ::    n, step, tstep, m, option, precision, gp
  integer   ::    i, j, contador
  !
  !       Lee parametros de archivo 
  open(1,file='diff.par')
  read(1,*)  n                   !   puntos de grilla
  read(1,*)  rmin
  read(1,*)  rmax
  read(1,*)  k                   !   coeficiente de difusion
  read(1,*)  alfa                !   dejar alfa=1
  read(1,*)  beta                !   por ahora no importa porque no lo usa
  read(1,*)  option              !   esquema temporal
  read(1,*)  precision           !   precision en el orden de la derivada
  read(1,*)  dt                  !   paso temporal
  read(1,*)  step                !   cantidad total de pasos
  read(1,*)  tstep               !   cadencia de grabacion
  close(1)
  !
  !-----------------------------------------------------------------------
  !aloja el espacion en memoria para cada vector
  allocate (F(n), DF(n), DDF(n), r(n))
  !-----------------------------------------------------------------------
  !
  !-------------------------------------------------------------------
  !     Determino la cantidad de puntos fantasmas segun precision
  if(precision.eq.1) then 
     gp=1
  endif
  if(precision.eq.2) then 
     gp=2
  endif
  !-------------------------------------------------------------------
  !       Calcula la cantidad de pasos temporales
  print*,'pasos de integracion temporal=  ',step
  !
  !-------------------------------------------------------------------
  !     Condiciones iniciales y de contorno, tambien construye 
  !     la grilla espacial
  call inic(n,gp,option,precision,rmin,rmax,r,F,DF,DDF)
!------------------------------------------------------------------------
! NOTA: dejo la generalizacion para distintos esquemas temporales 
! incompleta, pero dejo una idea de como podria que ser.
!------------------------------------------------------------------------
  !----------------------------------------------------------------------
  ! revisa que los valores de alfa y beta cumplan la condiciond de
  ! consistencia, en caso que no lo haga, avisa e interrumpe la ejecucion
  !----------------------------------------------------------------------
!  print*, 'alfa = ',alfa
!  if (alfa.gt.1).or.(alfa.lt.0) then
!10  print*,'El problema no preserva la condicion de consistencia'
!     stop
!  endif
!  sum=alfa+beta
!  if (sum.gt.1) goto 10
!  print*, 'beta = ',beta
!  if (beta.eq.0) then
!     print*,'eligio un esquema temporal explicito'
!  else
!     print*,'eligio un esquema temporal implicito'
!  endif
  !------------------------------------------------------------------
  ! graba la condicion inicial
  contador=1
  call record(n,r,F,contador)
  !------------------------------------------------------------------
  !     Comienza la parte principal del codigo
  !------------------------------------------------------------------
  do j=1,step
     if(m.eq.tstep) then
        call record(n,r,F,contador)
        m=0
     endif
     m=m+1   
     if(precision.eq.1) then
        !     call der1(n,gp,r,F,DF)
        call der2(n,gp,r,F,DDF)
     endif
     if(precision.eq.2) then
        !     call der1_5p(n,gp,r,F,DF)
        call der2_5p(n,gp,r,F,DDF)
     endif
     !Nota: si se elije otro esquema temporal hay que modificar esta
     !linea, en caso de un esquema implicito (beta neq 0), ademas hay 
     !que definir vectores auxiliares.
     do i=1,n
        F(i)=F(i)+dt*k*DDF(i)
     end do
  end do
  print*,'done'
end program diff
!*************************************************************************
!     Subroutines
!*************************************************************************
!
!*******************************************************************
subroutine inic(n,gp,option,precision,rmin,rmax,r,F,DF,DDF)
  !*******************************************************************
  !     Condiciones de contorno 
  !     Puntos de la region pasiva (1 y n, ghost zone)
  !     n(in) y n(out):informacion del problema
  !     meto las condiciones iniciales de velocidad y grilla
  !--------------------------------------------------------------------
  implicit none
  integer, intent (in) :: n
  integer, intent (in) :: gp
  integer, intent (in) :: option
  integer, intent (in) :: precision
  real, intent (in) :: rmin
  real, intent (in) :: rmax
  real, intent (out), dimension(*) :: r
  real, intent (out), dimension(*) :: F
  real, intent (out), dimension(*) :: DF
  real, intent (out), dimension(*) :: DDF
  real :: dr
  integer :: i
  !-------------------------------------------------------------------
  !inicializa la grilla para la integracion espacial
  dr = (rmax-rmin)/dfloat(n-1)
  do i = 2,n-1
     r(i) = rmin+dfloat(i-1)*dr
  end do
  r(1)=rmin
  r(n)=rmax
  do i=2,n-1
     F(i) = 0.
  end do
  F(1)=0.
  F(n)=100.
  DF(1)=0.0
  DF(n)=0.0
  if(precision.eq.2) then
     DF(2)=0.0
     DF(n-1)=0.0
  endif
  DDF(1)=0.0
  DDF(n)=0.0
  if(precision.eq.2) then
     DDF(2)=0.0
     DDF(n-1)=0.0
  endif
  return
end subroutine inic
!************************************************************************
! subrutina para grabar datos
!---------------------------------------------------------------------
subroutine record(n,x,F,cont)
  implicit none
  integer, intent (in) :: n
  integer, intent (inout) :: cont
  real, intent(in), dimension(*) :: x, F
  integer :: i, j, uni,dec,cen
  character (len=9) :: name
  character (len=3) :: auxc
  cen=cont/100
  dec=(cont-cen*100)/10
  uni=cont-cen*100-dec*10
  auxc=char(cen+48)//char(dec+48)//char(uni+48)
  name='F.'//auxc//'.dat'
  open(1,file=name,status='unknown')
  do i =1,n
        write(1,*) x(i), F(i)
  end do
  close(1)
  cont=cont+1
  return
end subroutine record
!--------------------------------------------------------------------

